#!/usr/bin/env python3
# Copyright 2023 - Barcelona Supercomputing Center
# Author: Rodrigo Martín Posada (2021), updated by : Amirpasha Mozaffari (2023) 
# TBD

import yaml
from yaml.loader import SafeLoader
import logging
import argparse
import os

import f90nml
import pyoasis
from   pyoasis import OASIS  # pylint: disable=no-name-in-module
import numpy as np
import time


# ++++++++++++++++
# OASIS PUT
# ++++++++++++++++ 
def run_oasis_put():
    logging.info('Entering OASIS_PUT')
    logging.debug('ITAP_SEC: {} sending fields'.format(itap_sec))
    logging.debug(str(out_coupling_fields.keys()))
    for id, field in out_coupling_fields.items():
        sigma=1
        mu=1
        # send dummy data (100 for runoff, random for other variables)
        #TODO send dummy data read from oasis restart file
        if id == "A_Runoff":
            field_data[:]=1.4658e-06
            field_data[:]=100
        else:
            field_data[:]=sigma * np.random.randn(local_size) + mu
        field.put(itap_sec, field_data)      
        logging.debug('{} put -> {:.3e} {:.3e} {:.3e}'.format( id, np.min(field_data), np.mean(field_data), np.max(field_data)))
    logging.debug('--------------------------------------\n')


# ++++++++++++++++
# OASIS GET
# ++++++++++++++++
def run_oasis_get():
    logging.info('Entering OASIS_GET')
    logging.debug('ITAP_SEC: {} receiving fields'.format(itap_sec))
    logging.debug(str(in_coupling_fields.keys()))
    for id, field in in_coupling_fields.items():
        field.get(itap_sec, field_data)      
        logging.debug('{} -> {:.3e} {:.3e} {:.3e}'.format( id, np.min(field_data), np.mean(field_data), np.max(field_data)))
    logging.debug('--------------------------------------\n')



if __name__ == '__main__':

### Read in YAML file 
    current_path = os.getcwd()
    parser=argparse.ArgumentParser()
    parser.add_argument("--yaml_file_path",type=str)
    args = parser.parse_args()
    yamel_file_path = os.path.join(current_path,args.yaml_file_path)
    if os.path.isfile(yamel_file_path) == False: 
        print("YAML file does not exist. Aborted")
        raise NameError("YAML file does not exist.")
        
    # Open the file and load the file 
    with open(yamel_file_path) as f:
        read_in_data = yaml.load(f, Loader=SafeLoader)
        f.close()

### Read in file 
    #read_yaml(read_in_data)
    model = read_in_data.get("model")
    model_name = model.get("model_name")
    model_nproc = model.get("model_nproc")
    model_scalability = model.get("scalability")
    
    # Read in Simulation information
    simulation = read_in_data.get("simulation")
    simulation_coupling_interval = simulation.get("coupling_interval")
    simulation_run_length_sec = simulation.get("run_length_sec")
    simulation_nx = simulation.get("nx")

    # Read in Coupling infomration
    coupling = read_in_data.get("coupling")
    coupling_out_vars = coupling.get("out_vars")
    coupling_in_vars = coupling.get("in_vars")
    restart_file = coupling.get("restart_file")

### Pass the values to original code 
    
    # Init OASIS
    comp = pyoasis.Component(model_name)
    local_comm = comp.localcomm
    # Get rank in local communicator
    mype = comp.localcomm.rank
    npes = comp.localcomm.size  
    
    # BUG : shouldnt it be the npes and not mpye (?)
    mype_value = str(100+mype)
    out_model = '{model_name}.out_{mype_value}'.format(model_name = model_name, mype_value = mype_value)

    logging.basicConfig(filename=out_model, format='%(message)s', level=logging.DEBUG)
    logging.debug('-----------------------------------------------------------')
    logging.info('I am {model_name} process with rank : {mype}'.format(model_name = model_name,
                                                                         mype = mype))
    #logging.debug('I am IFS-toy process with rank : {}'.format(mype))
    logging.debug('in my local communicator gathering {} processes'.format(npes))
    logging.debug('----------------------------------------------------------')

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    #  PARTITION DEFINITION
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    # Definition of the local partition
    local_size = simulation_nx // npes
    remainder = simulation_nx % npes

    if mype < remainder:
        local_size += 1
        offset = mype * local_size
    else:
        offset = mype * local_size + remainder

    logging.debug('MPI rank is {}, Local size is {}, Offset is {}'.format(mype, local_size, offset))
    partition = pyoasis.ApplePartition(offset, local_size)


    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    #  DECLARATION OF THE COUPLING FIELDS
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    out_coupling_fields={}

    for v in coupling_out_vars:
        out_coupling_fields[v] = pyoasis.Var(v, partition, OASIS.OUT)
        logging.info('var_id {}, added as OASIS_OUT var'.format(v))


    in_coupling_fields={}

    for v in coupling_in_vars:
        in_coupling_fields[v] = pyoasis.Var(v, partition, OASIS.IN)
        logging.info('var_id {}, added as OASIS_IN var'.format(v))


    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    #  DEFINE THE SPEED OF THE COMPONENT
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # If the number of processes from the YAML configuraion is not defined (or 0), use the communcator size
    if model_nproc is None or model_nproc == 0:
        model_nproc = npes
        if mype == 0:
            logging.info('WARNING: Model {} nproc was not defined in the YAML file. Using the communicator size instead ({} processes)'.format(model_name, model_nproc))
    # Read the scalaiblity properties of the component
    arr = np.genfromtxt(model_scalability, delimiter=",", skip_header=1)
    idx_closest_npes = np.argmin(abs((arr[:,0] - model_nproc)))
    sypd = arr[idx_closest_npes, 1]
    
    num_ts_in_one_sy = 365*24*3600/2700
    num_ts_executed = sypd * num_ts_in_one_sy
    runtime = 24*3600  # 1 day of execution
    timestep_length = runtime / num_ts_executed

    if mype == 0:
        print('Model {} is using {} processes and runs at {} SYPD. The timestep_length is {:.4f}.'.format(model_name, model_nproc, sypd, timestep_length))

    logging.info('nproc: {}'.format(model_nproc))
    logging.info('SYPD: {}'.format(sypd))
    logging.info('timestep_length: {}'.format(timestep_length))
    

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    #         TERMINATION OF DEFINITION PHASE
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    logging.debug('End of initialisation phase')

    comp.enddef()

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    #  TIME STEP LOOP
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    logging.debug('Timestep, field min, mean and max value')

    field_data = pyoasis.asarray(np.full(local_size, -1.0))

    for itap_sec in range(0, simulation_run_length_sec, simulation_coupling_interval):
        logging.info('timestep: {}'.format(itap_sec))
	    # Call OASIS_GET
        run_oasis_get()
        # Simulate component computation (sleep)
        time.sleep(timestep_length)
        # Call OASIS_PUT
        run_oasis_put()

    logging.debug('End of the program')

    if mype == 0:
        print('Model {} is finished successfully.'.format(model_name))

    del comp

