set -xuve

HPCROOTDIR=%HPCROOTDIR%
EXPID=%EXPID%
CHUNK=%CHUNK%
SDATE=%SDATE%
MEMBER=%MEMBER%
CHUNKSIZEUNIT=%CHUNKSIZEUNIT%
CHUNKSIZE=%CHUNKSIZE%

# Load the modules
module purge
module load intel/2017.4 mkl/2017.4 python/3.6.1 netcdf/4.2 impi/2017.4

# setup pyOasis environment
source $HPCROOTDIR/python-amip-reader/sources/oasis3-mct/generated/python/init.sh

# Check that inidata dir exists and is not empty, rise error and exit otherwise
inidata=${HPCROOTDIR}/${SDATE}/${MEMBER}/inidata
if [ ! -d ${inidata} ]; then
    echo "ERROR: ${inidata} does not exist"
    exit 1
fi
if [ ! -n "$(ls -A ${inidata})" ]; then
    echo "ERROR: ${inidata} is empty"
    exit 1
fi

# Set the number of components based on the count of YAML config files in the directory
number_of_components=$(ls -1 ${inidata}/COMP*_toy_model.yaml | wc -l)

# Create the member runtime directory if it is the first Chunk
runtime=${HPCROOTDIR}/${SDATE}/${MEMBER}/runtime
if [ ${CHUNK} -eq 1 ]; then
    # Remove old runtime if exists and create a new one
    if [ -d ${runtime} ]; then
        rm -rf ${runtime}
    fi
    mkdir -p ${runtime}
# If not the first Chunk, check that runtime exists. Print error and exit otherwise
else
    if [ ! -d ${runtime} ]; then
        echo "ERROR: ${runtime} does not exist"
        exit 1
    fi
fi

# Copy the initial data files from inidata to runtime
cp -r ${inidata}/* ${runtime}

# Go to the rundir
cd ${runtime}

# Print the values of the environment variables before the loop
for ((i=1; i<=number_of_components; i++)); do
    nproc_var="MY_COMP${i}_NPROC"
    echo "$nproc_var: ${!nproc_var}"
done

# Declare an array to store YAML filenames for each component
declare -a component_yaml_files

# Define the number of processors for each component from the environment variables
total_nproc=0
for ((i=1; i<=number_of_components; i++)); do
    yaml_file="${inidata}/COMP${i}_toy_model.yaml"
    component_yaml_files+=("${yaml_file}")

    # Substitute the number of processors in the yaml file
    nproc_var="MY_COMP${i}_NPROC"
    nproc="${!nproc_var}"
    sed -i "s/model_nproc: .*/model_nproc: ${nproc}/" "${yaml_file}"
    total_nproc=$((total_nproc+nproc))
done

# Modify the runtime length
# Change CHUNKSIZEUNIT "hour" "day" "week" or "month" to the corresponding number of seconds
if [ ${CHUNKSIZEUNIT} == "hour" ]; then
    CHUNKSIZEUNIT_seconds=3600
elif [ ${CHUNKSIZEUNIT} == "day" ]; then
    CHUNKSIZEUNIT_seconds=86400
elif [ ${CHUNKSIZEUNIT} == "week" ]; then
    CHUNKSIZEUNIT_seconds=604800
elif [ ${CHUNKSIZEUNIT} == "month" ]; then
    CHUNKSIZEUNIT_seconds=2592000
fi
# Run length time is the CHUNKSIZE multiplied by the number of chunks
sim_length_seconds=$((CHUNKSIZE*CHUNKSIZEUNIT_seconds))

# Change the run_length_sec in the yaml files for the value in sim_length_seconds
for yaml_config_file in ${component_yaml_files[@]}; do
    sed -i "s/run_length_sec: .*/run_length_sec: ${sim_length_seconds}/" ${yaml_config_file}
done
# Change the run_length_sec in the namcouple file
line_number=$(grep -n "RUNTIME" namcouple | cut -d: -f1)
next_line_number=$((line_number+1))
sed -i "${next_line_number}s/.*/   ${sim_length_seconds}/" namcouple

# Prepare the mpirun command
mpirun_command="mpirun"
for yaml_config_file in ${component_yaml_files[@]}; do
    mpirun_command+=" -np 1 python3 toy_model.py --yaml_file_path ${yaml_config_file} : "
done
# Remove the trailing colon from mpirun command
mpirun_command=${mpirun_command% }
mpirun_command=${mpirun_command%:}

# Run the model (in sequential)
t0=$(date +%s)
eval ${mpirun_command}
t1=$(date +%s)

# Calculate the execution time and SYPD
runtime_seconds=$((t1-t0))
echo "Runtime: ${runtime_seconds} seconds"
echo "Total number of processors: ${total_nproc}"
sypd=$(echo "scale=2; ${sim_length_seconds}/365/${runtime_seconds}" | bc)
echo "SYPD: ${sypd}"
# get the CHSY by multiplying the number of processors by 24 and dividing by the SYPD
chsy=$(echo "scale=2; ${total_nproc}*24/${sypd}" | bc)
echo "CHSY: ${chsy}"

# Move the output files to the runtime/log/chunk directory
log_dir=${runtime}/log/$(printf "%03d" ${CHUNK})
mkdir -p ${log_dir}

# Move the output files to the log directory
mv timeline_* debug.* load_balancing_info.txt nout.000000 *.out* ${log_dir}